@@ -25,6 +25,12 @@ ALL_GROUPS = '__groups__'
 ALL_ROWS = '__rows__'
 """Denotes every row."""
 
+FIELDS_STACKED = '__stacked__'
+"""Denotes stacked fields when layout is applied.
+E.g. in 'group': {'a', ['b', 'c']} b and c are stacked.
+
+"""
+
 FORM = '__form__'
 """Denotes a form."""
 
@@ -55,6 +61,12 @@ class FormComposer:
     opt_render_help: bool = True
     """Render hints (help texts)."""
 
+    opt_title_label: bool = False
+    """Render label as title for form field."""
+
+    opt_title_help: bool = False
+    """Render help as title for form field."""
+
     opt_placeholder_label: bool = False
     """Put title (verbose name) into field's placeholders."""
 
@@ -98,7 +110,8 @@ class FormComposer:
         ALL_FIELDS: '<span>{field}</span>',
         ALL_ROWS: '<div {attrs}>{fields}</div>',
         ALL_GROUPS: '<fieldset {attrs}><legend>{title}</legend>{rows}</fieldset>',
-        SUBMIT: '{submit}'
+        SUBMIT: '{submit}',
+        FIELDS_STACKED: '<div>{field}</div>',
     }
     """Wrappers for fields, groups, rows, submit button."""
 
@@ -217,12 +230,28 @@ class FormComposer:
         placeholder = attrs.get('placeholder')
 
         if placeholder is None:
+
             if self.opt_placeholder_label:
                 attrs['placeholder'] = field.label
 
             elif self.opt_placeholder_help:
                 attrs['placeholder'] = field.help_text
 
+        title = attrs.get('title')
+
+        if title is None:
+
+            title_label = self.opt_title_label
+            title_help = self.opt_title_help
+
+            if title_label or title_help:
+
+                if title_label:
+                    attrs['title'] = field.label
+
+                elif title_help:
+                    attrs['title'] = field.help_text
+
         out = field.as_widget(attrs=attrs)
 
         if field.field.show_hidden_initial:
@@ -336,27 +365,56 @@ class FormComposer:
 
         return self._apply_wrapper(fld=field, content=out)
 
-    def _render_group(self, alias: str, *, rows: List[str]) -> str:
-        title = self.groups.get(alias, '')
+    def _render_group(self, alias: str, *, rows: List[Union[BoundField, List[BoundField]]]) -> str:
 
         get_attrs = self._attrs_get
         attrs = self.attrs
         wrappers = self.wrappers
         format_value = self._format_value
+        render_field_box = self._render_field_box
 
         def get_group_params(container: dict) -> dict:
             get_params = partial(get_attrs, container)
             return {**get_params(ALL_GROUPS), **get_params(alias)}
 
+        def render(field: Union[BoundField, List[Union[BoundField, str]]], wrap: bool = False) -> str:
+
+            if isinstance(field, list):
+                out = []
+
+                for subfield in field:
+
+                    if isinstance(subfield, list):
+                        rendered = render(subfield, wrap=len(subfield) > 1)
+
+                    elif isinstance(subfield, str):
+                        rendered = subfield
+
+                    else:
+                        rendered = render_field_box(subfield)
+
+                    out.append(rendered)
+
+                out = '\n'.join(out)
+                if wrap:
+                    out = format_value(wrapper_stacked, field=out)
+
+                return out
+
+            return render_field_box(field)
+
+        wrapper_stacked = get_attrs(wrappers, FIELDS_STACKED)
+        wrapper_rows = get_attrs(wrappers, ALL_ROWS)
+
         html = format_value(
             get_group_params(wrappers),
             attrs=flatatt(get_group_params(attrs)),
-            title=title,
+            title=self.groups.get(alias, ''),
             rows='\n'.join(
                 format_value(
-                    get_attrs(wrappers, ALL_ROWS),
+                    wrapper_rows,
                     attrs=flatatt(get_attrs(attrs, ALL_ROWS, obj=fields)),
-                    fields='\n'.join(fields),
+                    fields=render(fields),
                 )
                 for fields in rows
             )
@@ -365,23 +423,21 @@ class FormComposer:
         return html
 
     def _render_layout(self) -> str:
-        render_field_box = self._render_field_box
         form = self.form
 
-        fields = {name: render_field_box(form[name]) for name in form.fields}
+        fields = {name: form[name] for name in form.fields}
 
         form_layout = self.layout[FORM]
 
-        out = [
-            self._render_feedback_nonfield(),
-        ]
+        out = []
 
         if isinstance(form_layout, str):
             # Simple layout defined by macros.
 
             if form_layout == ALL_FIELDS:
                 # all fields, no grouping
-                out.extend(fields.values())
+                render_field_box = self._render_field_box
+                out.extend(render_field_box(field) for field in fields.values())
 
             else:  # pragma: nocover
                 raise ValueError(f'Unsupported form layout macros: {form_layout}')
@@ -392,11 +448,13 @@ class FormComposer:
             grouped = defaultdict(list)
 
             def add_fields_left():
-                grouped[group_alias].extend([[field] for field in fields.values()])
+                group.extend([[field] for field in fields.values()])
                 fields.clear()
 
             for group_alias, rows in form_layout.items():
 
+                group = grouped[group_alias]
+
                 if isinstance(rows, str):
                     # Macros.
 
@@ -418,24 +476,32 @@ class FormComposer:
 
                             else:
                                 # One field in row.
-                                grouped[group_alias].append([fields.pop(row)])
+                                group.append([fields.pop(row)])
 
                         else:
-                            # Several fields in row.
-                            row_fields = [fields.pop(group_field, None) for group_field in row]
-                            grouped[group_alias].append(row_fields)
+                            # Several fields in a row.
+                            row_items = []
+                            for row_item in row:
+                                if not isinstance(row_item, list):
+                                    row_item = [row_item]
+                                row_items.append([fields.pop(row_subitem, '') for row_subitem in row_item])
+                            group.append(row_items)
 
             for group_alias, rows in grouped.items():
                 out.append(render_group(group_alias, rows=rows))
 
+        out.insert(0, self._render_feedback_nonfield())
+
         return '\n'.join(out)
 
     def _render_submit(self) -> str:
         get_attr = self._attrs_get
         return self._format_value(
             get_attr(self.wrappers, SUBMIT),
-            submit=f'<button type="submit" name="{self.opt_submit_name}" value="{self.form._submit_value}"'
-                   f'{flatatt(get_attr(self.attrs, SUBMIT))}>{self.opt_submit}</button>'
+            submit=(
+                f'<button type="submit" name="{self.opt_submit_name}" value="{self.form._submit_value}"'
+                f'{flatatt(get_attr(self.attrs, SUBMIT))}>{self.opt_submit}</button>'
+            )
         )
 
     def render(self) -> str:
@@ -445,14 +511,20 @@ class FormComposer:
 
         if self.opt_render_form:
             get_attr = partial(self._attrs_get, self.attrs)
-            request = self.form.request
+            form = self.form
+
+            form_id = form.id or ''
+            if form_id:
+                form_id = f' id="{form.id}"'
+
+            request = form.request
 
             csrf = ''
             if request:
                 csrf = f'<input type="hidden" name="csrfmiddlewaretoken" value="{get_token(request)}">'
 
             html = (
-                f"<form {flatatt(get_attr(FORM))}>"
+                f'<form {flatatt(get_attr(FORM))}{form_id}>'
                 f"{csrf}"
                 f"{html}"
                 f"{self._render_submit()}"
